@@ -19,7 +19,7 @@ along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
  *
- $Source: /home/bas/distr/ftape-2.02/RCS/fc-10.h,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/fc-10.h,v $
  $Author: bas $
  *
  $Revision: 1.3 $
