@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,21 +15,27 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/fdc-io.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/fdc-io.c,v $
  $Author: bas $
  *
- $Revision: 1.40 $
- $Date: 1995/01/08 14:16:21 $
+ $Revision: 1.47 $
+ $Date: 1995/05/27 08:54:21 $
  $State: Beta $
  *
  *      This file contains the low-level floppy disk interface code
  *      for the QIC-40/80 tape streamer device driver.
  */
 
-static char RCSid[] = "$Id: fdc-io.c,v 1.40 1995/01/08 14:16:21 bas Beta $";
+static char RCSid[] = "$Id: fdc-io.c,v 1.47 1995/05/27 08:54:21 bas Beta $";
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/errno.h>
 #include <linux/sched.h>
+#include <linux/ioport.h>
 #include <asm/system.h>
 #include <asm/io.h>
 #include <asm/dma.h>
@@ -42,7 +48,6 @@ static char RCSid[] = "$Id: fdc-io.c,v 1.40 1995/01/08 14:16:21 bas Beta $";
 #include "ftape-rw.h"
 #include "calibr.h"
 #include "fc-10.h"
-#include "kernel-version.h"
 
 
 /*      Global vars.
@@ -64,13 +69,14 @@ volatile byte fdc_cyl;          /* FDC track */
 volatile byte fdc_sect;         /* FDC sector */
 static int fdc_data_rate = 0;   /* default rate = 500 Kbps */
 static int fdc_seek_rate = 14;  /* default rate = 2 msec @ 500 Kbps */
-#if defined FDC_BASE || !defined FLOPPY_HACK
+#if defined FDC_BASE || !FLOPPY_HACK
 static void (*do_ftape)(void);
 #endif
 static int fdc_fifo_state;      /* original fifo setting - fifo enabled */
 static int fdc_fifo_thr;        /* original fifo setting - thresshold */
 static int fdc_lock_state;      /* original lock setting - locked */
 static int fdc_fifo_locked = 0; /* has fifo && lock set ? */
+static byte fdc_precomp = 0;    /* sets fdc to default precomp. value */
 
 
 void
@@ -313,6 +319,7 @@ fdc_issue_command( byte *out_data, int out_count,
 }
 
 /*      Wait for FDC interrupt with timeout.
+ *      Signals are blocked so the wait will not be aborted.
  *      Note: interrupts must be enabled ! (23/05/93 SJL)
  */
 int
@@ -329,51 +336,50 @@ fdc_interrupt_wait( int time) {
     return -EIO;                /* return error... */
   }
   if (interrupt_seen == 0) {
-    int signal_pending = 0;
-
-   /*   timeout time will be between 0 and MSPT milliseconds too long ! */
+    /* timeout time will be between 0 and MSPT milliseconds too long !
+     */
     current->timeout = jiffies + 1 + (time + MSPT - 1) / MSPT;
     current->state = TASK_INTERRUPTIBLE;
-  
+    current->blocked = _BLOCK_ALL;
     add_wait_queue( &wait_intr, &wait);
-  
-    while (1) {
-      while (!interrupt_seen && current->state != TASK_RUNNING) {
-        schedule();             /* sets TASK_RUNNING on timeout */
-      }
-      if (current->signal & ~current->blocked) {
-        /* awoken by signal, block it to be able to finish the wait
-         * but remember that we got a signal and must abort.
-         */
-        current->blocked |= (current->signal & ~current->blocked) ;
-        signal_pending = 1;
-      } else {
-        break;
-      }
-    }
+    do {
+      schedule();               /* sets TASK_RUNNING on timeout */
+    } while (!interrupt_seen && current->state != TASK_RUNNING);
+    current->blocked = current_blocked; /* restore */
+    remove_wait_queue( &wait_intr, &wait);
     if (interrupt_seen) {
-      /* got interrupt: good */
       current->timeout = 0;     /* interrupt hasn't cleared this */
       result = 0;
     } else {
-      if (signal_pending) {
-        /* didn't get interrupt but got signal (& timeout): abort */
-        current->timeout = 0;   /* interrupt hasn't cleared this */
+#if 1
+      /*** remove me when sure this doesn't happen ***/
+      if (current->timeout > 0) {
+        TRACE( -1, "*** BUG: unexpected schedule exit ***");
+        if (current->signal & ~current->blocked) {
+          TRACE( 4, "caused by signal ?");
+        }
+      }
+#endif
+      if (current->signal & ~current->blocked) {
         result = -EINTR;
       } else {
-        /* didn't get interrupt nor signal: timeout */
         result = -ETIME;
       }
-      need_cleanup = 1;
+      need_cleanup = 1;         /* missing interrupt, reset fdc. */
     }
-    remove_wait_queue( &wait_intr, &wait);
   } else {
     result = 0;
   }
-  current->blocked = current_blocked; /* restore old state */
+  /*  In first instance, next statement seems unnecessary since
+   *  it will be cleared in fdc_command. However, a small part of
+   *  the software seems to rely on this being cleared here
+   *  (ftape_close might fail) so stick to it until things get fixed !
+   */
   interrupt_seen = 0;		/* clear for next call */
+
   if (need_cleanup & !resetting) {
     resetting = 1;              /* break infinite recursion if reset fails */
+    TRACE( 8, "cleanup reset");
     fdc_reset();
     resetting = 0;
   }
@@ -406,19 +412,43 @@ fdc_motor( int motor)
   TRACE_EXIT;
 }
 
+static void
+fdc_update_dsr( void)
+{
+  TRACE_FUN( 8, "fdc_update_dsr");
+
+  TRACEx2( 5, "rate = %d, precomp = %d", fdc_data_rate, fdc_precomp);
+  if (fdc.type >= i82077) {
+    outb_p( (fdc_data_rate & 0x03) | fdc_precomp, fdc.dsr);
+  } else {
+    outb_p( fdc_data_rate, fdc.ccr);
+  }
+  TRACE_EXIT;
+}
+
+void
+fdc_set_write_precomp( int precomp)
+{
+  /*  write precompensation can be set in multiples of 41.67 nsec.
+   *  round the parameter to the nearest multiple and convert it
+   *  into a fdc setting. Note that 0 means default to the fdc,
+   *  7 is used instead of that.
+   */
+  fdc_precomp = ((precomp + 21) / 42) << 2;
+  if (fdc_precomp == 0) {
+    fdc_precomp = 7 << 2;
+  }
+  fdc_update_dsr();
+}
+
 void
 fdc_set_data_rate( int rate)
 {
   /* Select clock for fdc, must correspond with tape drive setting !
    * This also influences the fdc timing so we must adjust some values.
    */
-  if (fdc.type >= i82077) {
-    outb_p( rate & 0x03, fdc.dsr); /* set rate only, keep precomp, reset
-                                      and powerdown bits at 0 */
-  } else {
-    outb_p( rate, fdc.ccr);     /* set rate */
-  }
   fdc_data_rate = rate;
+  fdc_update_dsr();
   fdc_set_seek_rate( fdc_seek_rate); /* re-adjust for changed clock */
 }
 
@@ -446,6 +476,7 @@ fdc_reset( void)
 #endif
   fdc_usec_wait( 10 /* usec */); /* delay >= 14 fdc clocks */
   fdc_ctl |= FDC_RESET_NOT;
+  fdc_mode = fdc_idle;
 #ifdef MACH2
   outb_p( fdc_ctl & 0x0f, fdc.dor);
   outb_p( fdc_ctl, fdc.dor2);
@@ -496,7 +527,7 @@ fdc_disable( void)
           TRACE( -1, "couldn't lock old state again");
         }
       }
-      TRACEx3( 3, "fifo restored to: %sabled, thresshold %d, %slocked",
+      TRACEx3( 5, "fifo restored: %sabled, thr. %d, %slocked",
               fdc_fifo_state ? "en" : "dis",
               fdc_fifo_thr, (fdc_lock_state) ? "" : "not ");
     }
@@ -593,7 +624,6 @@ fdc_seek( int track)
   byte out[3];
   int st0, pcn;
 
-  ftape_sleep( MILLISECOND);    /* NEEDED to prevent problems */
   out[ 0] = FDC_SEEK;
   out[ 1] = FTAPE_UNIT;
   out[ 2] = track;
@@ -815,7 +845,7 @@ fdc_fifo_enable( void)
       fdc_fifo_state = (reg[ 8] & 0x20) == 0;
       fdc_lock_state = reg[ 7] & 0x80;
       fdc_fifo_thr = 1 + (reg[ 8] & 0x0f);
-      TRACEx3( 3, "original fifo state: %sabled, thresshold %d, %slocked",
+      TRACEx3( 5, "original fifo state: %sabled, thresshold %d, %slocked",
               (fdc_fifo_state) ? "en" : "dis",
               fdc_fifo_thr, (fdc_lock_state) ? "" : "not ");
       /*  If fdc is already locked, unlock it first !
@@ -948,13 +978,13 @@ fdc_config_regs( unsigned fdc_base, unsigned fdc_irq, unsigned fdc_dma)
   fdc.tdr = fdc_base + 3;
   fdc.msr = fdc.dsr = fdc_base + 4;
   fdc.fifo = fdc_base + 5;
-#ifdef MACH2
+#if defined MACH2 || defined PROBE_FC10
   fdc.dor2 = fdc_base + 6;
 #endif
   fdc.dir = fdc.ccr = fdc_base + 7;
 }
 
-/*  If probing for a FC-10 controller the fdc base address, interrupt
+/*  If probing for a FC-10/20 controller the fdc base address, interrupt
  *  and dma channel must be specified.
  *  If using an alternate fdc controller, base address, interrupt and
  *  dma channel must be specified.
@@ -977,16 +1007,20 @@ fdc_config( void)
 
   if (!already_done) {
 #ifdef PROBE_FC10
+    int fc_type;
+
     fdc_config_regs( FDC_BASE, FDC_IRQ, FDC_DMA);
-    if (fc10_enable()) {
-      TRACE( 2, "FC-10 controller found");
+    fc_type = fc10_enable();
+    if (fc_type != 0) {
+      TRACEx1( 2, "FC-%c0 controller found", '0' + fc_type);
       fdc.type = fc10;
       fdc.hook = &do_ftape;
     } else {
-      TRACE( 2, "FC-10 controller not found");
+      TRACE( 2, "FC-10/20 controller not found");
       fdc.type = no_fdc;
+      fdc.dor2 = 0;             /* not used with std fdc */
       fdc_config_regs( 0x3f0, 6, 2); /* back to std fdc again */
-# ifdef FLOPPY_HACK
+# if FLOPPY_HACK
       fdc.hook = &do_floppy;
 # else
       fdc.hook = &do_ftape;
@@ -1000,7 +1034,7 @@ fdc_config( void)
 # else
     TRACE( 2, "Using the standard fdc controller");
     fdc_config_regs( 0x3f0, 6, 2); /* std fdc */
-#  ifdef FLOPPY_HACK
+#  if FLOPPY_HACK
     fdc.hook = &do_floppy;
 #  else
     fdc.hook = &do_ftape;
@@ -1013,9 +1047,13 @@ fdc_config( void)
   TRACE_EXIT;
 }
 
-#ifndef FLOPPY_HACK
+#if !FLOPPY_HACK
 
+#if KERNEL_VERSION < 1001082
 static void ftape_interrupt( int unused)
+#else
+static void ftape_interrupt( int irq, struct pt_regs* regs)
+#endif
 {
   TRACE_FUN( 8, "ftape_interrupt");
   void (*handler)(void) = *fdc.hook;
@@ -1036,13 +1074,13 @@ fdc_grab_irq_and_dma( void)
 {
   TRACE_FUN( 8, "fdc_grab_irq_and_dma");
   int result = 0;
-#ifndef FLOPPY_HACK
+#if !FLOPPY_HACK
   static char ftape_id[] = "ftape";
 
   if (fdc.hook == &do_ftape) {
     /*  Get fast interrupt handler.
      */
-#ifdef OLD_IRQ_REQUEST
+#if KERNEL_VERSION < 1001043
     struct sigaction ftape_sig_action = {
       ftape_interrupt, 0, SA_INTERRUPT, NULL
     };
@@ -1054,7 +1092,7 @@ fdc_grab_irq_and_dma( void)
       TRACEx1( -1, "Unable to grab IRQ%d for ftape driver", fdc.irq);
       result = -EIO;
     } else {
-#ifdef OLD_DMA_REQUEST
+#if KERNEL_VERSION < 1001048
       result = request_dma( fdc.dma);
 #else
       result = request_dma( fdc.dma, ftape_id);
@@ -1090,7 +1128,7 @@ fdc_release_irq_and_dma( void)
   TRACE_FUN( 8, "fdc_grab_irq_and_dma");
   int result = 0;
 
-#ifdef FLOPPY_HACK
+#if FLOPPY_HACK
   if (fdc.hook == &do_floppy) {
     do_floppy = NULL;           /* prevent accidents ! */
   }
@@ -1117,6 +1155,26 @@ fdc_release_irq_and_dma( void)
   return result;
 }
 
+int
+fdc_uninit( void)
+{
+  TRACE_FUN( 8, "fdc_uninit");
+  int result = 0;
+
+#if KERNEL_VERSION >= 1001080
+  if (fdc.sra != 0) {
+    if (fdc.dor2 == 0) {
+      release_region( fdc.sra, 6);
+      release_region( fdc.sra + 7, 1);
+    } else {
+      release_region( fdc.sra, 8);
+    }
+  }
+#endif
+  TRACE_EXIT;
+  return result;
+}
+
 int
 fdc_init( void)
 {
@@ -1131,7 +1189,7 @@ fdc_init( void)
     fdc_catch_stray_interrupts( 1); /* one always comes */
     TRACE( 5, "resetting fdc");
     fdc_reset();                  /* init fdc & clear track counters */
-    if (fdc.type == no_fdc) {     /* default, means no fc-10 found */
+    if (fdc.type == no_fdc) {     /* default, means no FC-10 or 20 found */
       fdc.type = fdc_probe();
     }
     if (fdc.type != no_fdc) {
@@ -1139,7 +1197,7 @@ fdc_init( void)
         if (fdc_fifo_enable() < 0) {
           TRACE( 2, "couldn't enable fdc fifo !");
         } else {
-          TRACE( 2, "fdc fifo enabled and locked");
+          TRACE( 5, "fdc fifo enabled and locked");
         }
       }
     } else {
@@ -1147,6 +1205,16 @@ fdc_init( void)
       result = -EIO;
     }
   }
+#if KERNEL_VERSION >= 1001080
+  if (result >= 0) {
+    if (fdc.dor2 == 0) {
+      request_region( fdc.sra, 6, "fdc (ftape)" );
+      request_region( fdc.sra + 7, 1, "fdc (ftape)" );
+    } else {
+      request_region( fdc.sra, 8, "fdc (ftape)" );
+    }
+  }
+#endif
   TRACE_EXIT;
   return result;
 }
