@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-ctl.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-ctl.c,v $
  $Author: bas $
  *
- $Revision: 1.2 $
- $Date: 1995/01/15 15:56:46 $
+ $Revision: 1.10 $
+ $Date: 1995/05/27 08:54:21 $
  $State: Beta $
  *
  *      This file contains the non-read/write ftape functions
@@ -27,10 +27,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-ctl.c,v 1.2 1995/01/15 15:56:46 bas Beta bas $";
+"$Id: ftape-ctl.c,v 1.10 1995/05/27 08:54:21 bas Beta $";
 
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/errno.h>
+#include <linux/mm.h>
+#include <asm/segment.h>
 
 #include "ftape.h"
 #include "ftape-eof.h"
@@ -40,6 +47,7 @@ static char RCSid[] =
 #include "ftape-read.h"
 #include "ftape-rw.h"
 #include "qic117.h"
+#include "ftape-bsm.h"
 
 
 /*      Global vars.
@@ -59,12 +67,13 @@ int no_tape = 1;
 int formatted = 0;
 int ftape_data_rate = 0;
 int going_offline = 0;
+int read_only = 0;
 
 /*      Local vars.
  */
 static int ftape_last_error = 0;
-static vendor_struct vendors[] = QIC117_VENDORS;
-static wakeup_method methods[] = WAKEUP_METHODS;
+static const vendor_struct vendors[] = QIC117_VENDORS;
+static const wakeup_method methods[] = WAKEUP_METHODS;
 static int init_drive_needed = 1;
 
 
@@ -156,9 +165,9 @@ ftape_abort_operation( void)
   int status;
 
   if (runner_status == running) {
-    TRACE( 5, "aborting running runner, waiting");
+    TRACE( 5, "aborting runner, waiting");
     runner_status = do_abort;
-    /* set timeout so that the tape will run into at logical EOT
+    /* set timeout so that the tape will run to logical EOT
      * if we missed the last sector and there are no queue pulses.
      */
     result = ftape_dumb_stop();
@@ -175,7 +184,7 @@ ftape_abort_operation( void)
     location.known = 0;
     runner_status = idle;
   }
-  for (i = 0; i < NR_ITEMS( buffer); ++i) {
+  for (i = 0; i < ftape_num_buffers; ++i) {
     buffer[ i].status = waiting;
   }
   head = tail = 0;
@@ -255,7 +264,18 @@ ftape_activate_drive( vendor_struct* drive_type)
     }
     for (method = no_wake_up; method < NR_ITEMS( methods); ++method) {
       drive_type->wake_up = method;
-      result = ftape_wakeup_drive( drive_type->wake_up);
+      /*  Test setup for dual drive configuration in dodo.
+       *  /dev/rft2 uses mountain wakeup only -> Archive QIC-80
+       *  /dev/rft3 uses colorado wakeup only -> Jumbo QIC-40
+       *  Other systems will use the normal scheme.
+       */
+      if (!DODO || (FTAPE_UNIT < 2) ||
+          (FTAPE_UNIT == 2 && method == wake_up_mountain) ||
+          (FTAPE_UNIT == 3 && method == wake_up_colorado)) {
+        result = ftape_wakeup_drive( drive_type->wake_up);
+      } else {
+        result = -EIO;
+      }
       if (result >= 0) {
         int tracing = old_tracing; /* fool TRACE */
         TRACEx1( 2, "drive wakeup method: %s",
@@ -303,13 +323,11 @@ ftape_get_drive_status( int* new_tape, int* no_tape, int* wp_tape)
     }
     /*  Clear error condition (drive is ready !)
      */
-    if (status & (QIC_STATUS_NEW_CARTRIDGE | QIC_STATUS_ERROR)) {
+    if (status & QIC_STATUS_ERROR) {
       int error;
       int command;
 
-      if (status & QIC_STATUS_ERROR) {
-        TRACE( 1, "error status set");
-      }
+      TRACE( 1, "error status set");
       result = ftape_report_error( &error, &command, 1);
       if (result < 0) {
         TRACEi( 1, "report_error_code failed:", result);
@@ -317,13 +335,24 @@ ftape_get_drive_status( int* new_tape, int* no_tape, int* wp_tape)
         init_drive_needed = 1;
         result = -EIO;
         break;
+      } else if (error != 0) {
+        TRACEi( 4, "error code   :", error);
+        TRACEi( 4, "error command:", command);
       }
-      TRACEi( 4, "error code   :", error);
-      TRACEi( 4, "error command:", command);
-      if (status & QIC_STATUS_NEW_CARTRIDGE) {
-        TRACE( 3, "status: new cartridge");
-        *new_tape = 1;
-      }
+    }
+    if (status & QIC_STATUS_NEW_CARTRIDGE) {
+      int error;
+      int command;
+      int old_tracing = tracing;
+
+      /*  Undocumented feature: Must clear (not present!) error 
+       *  here or we'll fail later.
+       */
+      tracing = 0;
+      ftape_report_error( &error, &command, 1);
+      tracing = old_tracing;
+      TRACE( 3, "status: new cartridge");
+      *new_tape = 1;
     }
   } while (status & QIC_STATUS_ERROR);
 
@@ -429,15 +458,10 @@ ftape_calc_timeouts( void)
     break;
   }
   if (tape_len <= 0) {
-    /*  handle variable length tapes as 400 ft (worst case)
-     *  I'm not sure yet but there seem to be two diffent
-     *  wide tapes: 300 ft and 400 ft.
-     *  Maybe the extended length indication works here too
-     *  so we can determine the actual length.
-     *  For now we keep a safe value.
+    /*  Handle unknown length tapes as 1100 ft ones (worst case)
      */
-    TRACE( 1, "Unknown tape length !");
-    length = 400;
+    TRACE( 1, "Unknown tape length, using worst case timing values!");
+    length = 1100;
   } else {
     length = tape_len;
   }
@@ -453,6 +477,7 @@ ftape_calc_timeouts( void)
     drive_type.speed = (2 * 12 * length * 1000) / dt;
     TRACE( -1, "==========================================");
     TRACEx1( -1, "drive : %s", drive_type.name);
+    TRACEx2( -1, "delta time = %d, length = %d", dt, length);
     TRACEx1( -1, "has max tape speed of %d ips", drive_type.speed);
     TRACE( -1, "please report this to <bas@vimec.nl>");
     TRACE( -1, "==========================================");
@@ -478,18 +503,15 @@ ftape_init_drive( int* formatted)
   int status;
 
   result = ftape_report_raw_drive_status( &status);
-  if (result >= 0) {
-    int recalibrate = 0;
-
+  if (result >= 0 && (status & QIC_STATUS_CARTRIDGE_PRESENT)) {
     if (!(status & QIC_STATUS_AT_BOT)) {
       /*  Antique drives will get here after a soft reset,
        *  modern ones only if the driver is loaded when the
        *  tape wasn't rewound properly.
        */
       ftape_seek_to_bot();
-      recalibrate = 1;
     }
-    if (recalibrate || !(status & QIC_STATUS_REFERENCED)) {
+    if (!(status & QIC_STATUS_REFERENCED)) {
       TRACE( 5, "starting seek_load_point");
       result = ftape_command_wait( QIC_SEEK_LOAD_POINT,
                                   timeout.reset, &status);
@@ -552,8 +574,14 @@ _ftape_open( void)
         if (drive_type.vendor_id == UNKNOWN_VENDOR) {
           ftape_log_vendor_id();
         }
-        if ((!no_tape && new_tape) || (!ftape_offline && init_drive_needed)) {
+        if (no_tape) {
+          ftape_offline = 1;
+        } else if (new_tape) {
           ftape_offline = 0;
+          init_drive_needed = 1;
+          read_only = 0;        /* enable writes again */
+        }
+        if (!ftape_offline && init_drive_needed) {
           result = ftape_init_drive( &formatted);
           if (result >= 0) {
             new_tape = 0;
@@ -577,46 +605,35 @@ int
 _ftape_close( void)
 {
   TRACE_FUN( 8, "_ftape_close");
-  int result;
-  int last_segment;
+  int result = 0;
+  int last_segment = 0;
 
-  result = ftape_flush_buffers();
-  last_segment = ftape_seg_pos - 1;
-  if (!(ftape_unit & FTAPE_NO_REWIND)) {
-    if (result >= 0) {
-      result = ftape_update_header_segments( NULL, 1);
-      if (result < 0) {
-        TRACE( 1, "error: update of header segments failed");
-      }   
-    } else {
-      TRACE( 1, "error: unable to update header segments");
+  if (!ftape_offline) {
+    result = ftape_flush_buffers();
+    last_segment = ftape_seg_pos - 1;
+    if (!(ftape_unit & FTAPE_NO_REWIND)) {
+      if (result >= 0) {
+        result = ftape_update_header_segments( NULL, 1);
+        if (result < 0) {
+          TRACE( 1, "error: update of header segments failed");
+        }   
+      } else {
+        TRACE( 1, "error: unable to update header segments");
+      }
     }
-  }
-  ftape_abort_operation();
-  if (!(ftape_unit & FTAPE_NO_REWIND)) {
-    if (!no_tape) {
-      TRACE( 5, "rewinding tape");
-      result = ftape_seek_to_bot();
+    ftape_abort_operation();
+    if (!(ftape_unit & FTAPE_NO_REWIND)) {
+      if (!no_tape) {
+        TRACE( 5, "rewinding tape");
+        result = ftape_seek_to_bot();
+      }
+      ftape_reset_position();
+      ftape_zap_read_buffers();
+      ftape_zap_write_buffers();
     }
-    ftape_reset_position();
-    ftape_zap_read_buffers();
-    ftape_zap_write_buffers();
   } 
-  /* Gnu tar stops on the EOF as it knows there is no more data
-   * and doesn't read the EOF, so inorder the next open doesn't start
-   * on an EOF we clear the state of eof_mark now.
-   */
-  if (just_before_eof) {        /* nothing was left */
-#if 0
-   /* 30/12/94 - this feature is disabled as it isn't compatible
-    *            with the way other tape drivers handle file marks.
-    *            (now it is compatible with the scsi driver)
-    */
-    eof_mark = 0;
-#endif
-    just_before_eof = 0;
-  }
   ftape_detach_drive();
+  fdc_uninit();
   if (history.used) {
     TRACE( 3,  "== Non-fatal errors this run: ==");
     TRACE( 3,  "fdc isr statistics:");
@@ -631,8 +648,7 @@ _ftape_close( void)
       TRACE( 3,  "ecc statistics:");
       TRACEi( 3, " crc_errors       :", history.crc_errors);
       TRACEi( 3, " crc_failures     :", history.crc_failures);
-      TRACEx2( 3, " ecc_failures     : %d%s", history.ecc_failures,
-              history.ecc_failures ? " !!!" : "");
+      TRACEi( 3, " ecc_failures     :", history.ecc_failures);
       TRACEi( 3, " sectors corrected:", history.corrected);
     }
     TRACEx2( 3, "media defects     : %d%s", history.defects,
@@ -675,14 +691,14 @@ _ftape_ioctl( unsigned int command, void * arg)
     }
     memcpy_fromfs( &krnl_arg.mtop, arg, arg_size);
   }
-  TRACEx1( -5, "called with ioctl command: 0x%08x", command);
+  TRACEx1( 5, "called with ioctl command: 0x%08x", command);
   switch (command) {
     /* cpio compatibility
      * mtrasx and mtreset are mt extension by Hennus Bergman
      * mtseek and mttell are mt extension by eddy olk
      */
   case MTIOCTOP:
-    TRACEx1( -5, "calling MTIOCTOP command: 0x%08x", krnl_arg.mtop.mt_op);
+    TRACEx1( 5, "calling MTIOCTOP command: 0x%08x", krnl_arg.mtop.mt_op);
     switch (krnl_arg.mtop.mt_op) {
     case MTNOP:
       /* gnu mt calls MTNOP before MTIOCGET to set status */
@@ -790,6 +806,9 @@ _ftape_ioctl( unsigned int command, void * arg)
         break;
       }
       result = ftape_weof( krnl_arg.mtop.mt_count, ftape_seg_pos, 1);
+      if (result >= 0) {
+        ftape_seg_pos += krnl_arg.mtop.mt_count - 1;
+      }
       break;
       /* MTRASx and MTRESET are mt extension by Hennus Bergman
        */
@@ -835,3 +854,50 @@ _ftape_ioctl( unsigned int command, void * arg)
   TRACE_EXIT;
   return result;
 }
+
+void
+ftape_init_driver( void)
+{
+  drive_type.vendor_id = UNKNOWN_VENDOR;
+  drive_type.speed = 0;
+  drive_type.wake_up = unknown_wake_up;
+  drive_type.name = "Unkown";
+
+  timeout.seek = 650 * SECOND;
+  timeout.reset = 670 * SECOND;
+  timeout.rewind = 650 * SECOND;
+  timeout.head_seek = 15 * SECOND;
+  timeout.stop = 5 * SECOND;
+  timeout.pause = 16 * SECOND;
+
+  qic_std = -1;
+  tape_len = -1;
+  current_command = 0;
+  current_cylinder = -1;
+
+  segments_per_track = 102;
+  segments_per_head = 1020;
+  segments_per_cylinder = 4;
+  tracks_per_tape = 20;
+  ftape_failure = 1;
+  ftape_seg_pos = 0;
+  first_data_segment = -1;
+  ftape_state = idle;
+  no_tape = 1;
+  formatted = 0;
+  ftape_data_rate = 0;
+  going_offline = 0;
+  read_only = 0;
+
+  init_drive_needed = 1;
+  header_segment_1 = -1;
+  header_segment_2 = -1;
+  used_header_segment = -1;
+  location.track = -1;
+  location.known = 0;
+  tape_running = 0;
+  might_be_off_track = 1;
+
+  ftape_new_cartridge();        /* init some tape related variables */
+  ftape_init_bsm();
+}
