@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-io.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-io.c,v $
  $Author: bas $
  *
- $Revision: 1.49 $
- $Date: 1995/01/15 13:18:58 $
+ $Revision: 1.58 $
+ $Date: 1995/05/27 08:54:21 $
  $State: Beta $
  *
  *      This file contains the general control functions
@@ -27,11 +27,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-io.c,v 1.49 1995/01/15 13:18:58 bas Beta $";
+"$Id: ftape-io.c,v 1.58 1995/05/27 08:54:21 bas Beta $";
 
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/errno.h>
 #include <linux/sched.h>
+#include <linux/mm.h>
 #include <asm/segment.h>
 #include <asm/system.h>
 #include <linux/ioctl.h>
@@ -52,42 +58,35 @@ static char RCSid[] =
 /*      Global vars.
  */
 /* NOTE: sectors start numbering at 1, all others at 0 ! */
-timeout_table timeout = {
-  650 * SECOND,  /* seek */
-  670 * SECOND,  /* reset */
-  650 * SECOND,  /* rewind */
-  15 * SECOND,   /* seek head */
-  5 * SECOND,    /* stop */
-  16 * SECOND,   /* pause */
-};
-vendor_struct drive_type = {
-  UNKNOWN_VENDOR, 0, unknown_wake_up, "Unknown"
-};
-int qic_std = -1;
-int tape_len = -1;
+timeout_table timeout;
+vendor_struct drive_type;
+int qic_std;
+int tape_len;
+volatile int current_command;
+const struct qic117_command_table qic117_cmds[] = QIC117_COMMANDS;
+int might_be_off_track;
 
 /*      Local vars.
  */
-static int check_restrictions = 1;
+static int command_parameter = 0;
 /*      command-restrictions is a table according
  *      to the QIC-117 specs specifying the state
  *      the drive status should be in at command execution.
  */
-struct qic117_command_table qic117_cmds[] = QIC117_COMMANDS;
-static ftape_error ftape_errors[] = QIC117_ERRORS;
+static const ftape_error ftape_errors[] = QIC117_ERRORS;
 static int ftape_udelay_count;
 static int ftape_udelay_time;
 static const struct {
   char* text;
   int fdc_code;
   byte drive_code;
+  int precomp;
 } rates[ 4] = {
-  { "2 M",   -1, /*unsupported*/ QIC_CONFIG_RATE_2000 },
-  { "1 M",   fdc_data_rate_1000, QIC_CONFIG_RATE_1000 },
-  { "500 K", fdc_data_rate_500,  QIC_CONFIG_RATE_500 },
-  { "250 K", fdc_data_rate_250,  QIC_CONFIG_RATE_250 },
+  { "2 M",   -1, /*unsupported*/ QIC_CONFIG_RATE_2000,   0},
+  { "1 M",   fdc_data_rate_1000, QIC_CONFIG_RATE_1000,  42},
+  { "500 K", fdc_data_rate_500,  QIC_CONFIG_RATE_500,  125},
+  { "250 K", fdc_data_rate_250,  QIC_CONFIG_RATE_250,  250},
 };
-static int interruptable = 1;
 typedef enum { prehistoric, pre_qic117c, post_qic117b, post_qic117d
              } qic_model;
 
@@ -153,83 +152,144 @@ int
 ftape_command( int command)
 {
   TRACE_FUN( 8, "ftape_command");
-  int result;
+  int result = 0;
   int track;
   int old_tracing = tracing;
+  static int level = 0;
+  int status = -1;
 
-  if (check_restrictions &&     /* only for commands, not parameters */
-      (command == QIC_REPORT_NEXT_BIT || command == QIC_REPORT_DRIVE_STATUS)) {
-    tracing = 0;                /* disable log of status call */
-  }
-  if (command <= 0 || command > NR_ITEMS( qic117_cmds)) {
-    TRACEx1( -1, "bug: bad command parameter: %d", command);
-    TRACE_EXIT;
-    return -EIO;
-  } else {
-    TRACEx1( 5, "called with command = %d", command);
-  }
-  /*  In fact restrictions are only checked for motion
-   *  commands and commands where the status must be `ready'.
-   */
-    /*  Conditions that next command may only be isued when the
-     *  drive is ready:
-     *  - motion command while motion command may be still busy.
-     *  - required status must be `ready'.
+  if (++level > 5) {
+    /*  This is a bug we'll want to know about.
      */
-  if (check_restrictions &&
-      ((!interruptable && !qic117_cmds[ command].interruptable) ||
-       (qic117_cmds[ command].state &
-        qic117_cmds[ command].mask & QIC_STATUS_READY))) {
-    int status;
-
-    result = ftape_report_raw_drive_status( &status);
-    if (result < 0) {
-      TRACE( 1, "ftape_report_raw_drive_status failed");
-    } else if ((status & qic117_cmds[ command].mask) ^
-               qic117_cmds[ command].state) {
-      TRACEx2( 1, "wrong state: 0x%02x should be: 0x%02x",
-              status & qic117_cmds[ command].mask,
-              qic117_cmds[ command].state);
-      result = ftape_ready_wait( timeout.seek, &status);
-    } else if (!interruptable && !qic117_cmds[ command].interruptable) {
-      /*  If !interruptable drive will not be ready or
-       *  else interruptable would be cleared.
+    TRACEx1( 1, "bug - recursion for command: %d", command);
+    result = -EIO;
+  } else if (command_parameter) {
+    /*  Don't check restrictions for parameters.
+     */
+    TRACEx1( 5, "called with parameter = %d", command - 2);
+  } else if (command <= 0 || command > NR_ITEMS( qic117_cmds)) {
+    /*  This is a bug we'll want to know about too.
+     */
+    TRACEx1( -1, "bug - bad command: %d", command);
+    result = -EIO;
+  } else {
+    /*  disable logging and restriction check for some commands,
+     *  check all other commands that have a prescribed starting status.
+     */
+    if (command == QIC_REPORT_DRIVE_STATUS) {
+      TRACE( 8, "report drive status called");
+      tracing = 0;
+    } else if (command == QIC_REPORT_NEXT_BIT) {
+      tracing = 0;
+    } else {
+      TRACEx1( 5, "%s", qic117_cmds[ command].name);
+      /*  A new motion command during an uninterruptible (motion)
+       *  command requires a ready status before the new command
+       *  can be issued. Otherwise a new motion command needs to
+       *  be checked against required status.
        */
-      TRACEx1( 1, "motion command %d issued while in motion", command);
-      result = ftape_ready_wait( timeout.seek, &status);
-    }
-    if (!(status & QIC_STATUS_READY)) {
-      result = -EBUSY;
-    }
-    if (result < 0) {
-      tracing = old_tracing;
-      TRACE_EXIT;
-      return result;
+      if (qic117_cmds[ command].cmd_type == motion &&
+          qic117_cmds[ current_command].non_intr) {
+        ftape_report_raw_drive_status( &status);
+        if ((status & QIC_STATUS_READY) == 0) {
+          TRACEx2( 4, "motion cmd (%d) during non-intr cmd (%d)",
+                  command, current_command);
+          TRACE( 4, "waiting until drive gets ready");
+          ftape_ready_wait( timeout.seek, &status);
+        }
+      }
+      if (qic117_cmds[ command].mask != 0) {
+        byte difference;
+
+        /*  Some commands do require a certain status:
+         */
+        if (status == -1) {     /* not yet set */
+          ftape_report_raw_drive_status( &status);
+        }
+        difference = ((status ^ qic117_cmds[ command].state) &
+                      qic117_cmds[ command].mask);
+        /*  Wait until the drive gets ready. This may last forever
+         *  if the drive never gets ready...
+         */
+        while ((difference & QIC_STATUS_READY) != 0) {
+          TRACEx1( 4, "command %d issued while not ready", command);
+          TRACE( 4, "waiting until drive gets ready");
+          ftape_ready_wait( timeout.seek, &status);
+          difference = ((status ^ qic117_cmds[ command].state) &
+                        qic117_cmds[ command].mask);
+          /*  Bail out on signal !
+           */
+          if (current->signal & _DONT_BLOCK) {
+            result = -EINTR;
+            break;
+          }
+        }
+        while (result == 0 && (difference & QIC_STATUS_ERROR) != 0) {
+          int err;
+          int cmd;
+
+          TRACEx1( 4, "command %d issued while error pending", command);
+          TRACE( 4, "clearing error status");
+          ftape_report_error( &err, &cmd, 1);
+          ftape_report_raw_drive_status( &status);
+          difference = ((status ^ qic117_cmds[ command].state) &
+                        qic117_cmds[ command].mask);
+          /*  Bail out on fatal signal !
+           */
+          if (current->signal & _DONT_BLOCK) {
+            result = -EINTR;
+            break;
+          }
+        }
+        if (result == 0 && difference) {
+          /*  Any remaining difference can't be solved here.
+           */
+          if (difference & (QIC_STATUS_CARTRIDGE_PRESENT |
+                            QIC_STATUS_NEW_CARTRIDGE |
+                            QIC_STATUS_REFERENCED)) {
+            TRACE( 1, "Fatal: tape removed or reinserted !");
+            ftape_failure = 1;
+          } else {
+            TRACEx2( 1, "wrong state: 0x%02x should be: 0x%02x",
+                    status & qic117_cmds[ command].mask,
+                    qic117_cmds[ command].state);
+          }
+          result = -EIO;
+        }
+        if (~status & QIC_STATUS_READY & qic117_cmds[ command].mask) {
+          TRACE( 1, "Bad: still busy!");
+          result = -EBUSY;
+        }
+      } 
     }
   }
   tracing = old_tracing;
-  check_restrictions = 1;       /* always turned on for next command */
-  if (qic117_cmds[ command].seeking) {
-    location.known = 0;         /* no longer valid after these commands */
-  }
-  if (!qic117_cmds[ command].interruptable) {
-    interruptable = 0;
-  }
-  /*
-   *    Keep cylinder nr within range, step towards home if possible.
+  /*  Now all conditions are met or result is < 0.
    */
-  if (current_cylinder >= command) {
-    track = current_cylinder - command;
-  } else {
-    track = current_cylinder + command;
-  }
-  result = fdc_seek( track);
-  /*    Verify whether we issued the right tape command.
-   */
-  if (result < 0) {
-    TRACE( 1, "fdc_seek failed");
-    result = -EIO;
+  if (result >= 0) {
+    /*  Always wait for a command_timeout period to separate
+     *  individuals commands and/or parameters.
+     */
+    ftape_sleep( 3 * MILLISECOND);
+    /*  Keep cylinder nr within range, step towards home if possible.
+     */
+    if (current_cylinder >= command) {
+      track = current_cylinder - command;
+    } else {
+      track = current_cylinder + command;
+    }
+    result = fdc_seek( track);
+    /*  position is no longer valid after any of these commands
+     *  have completed.
+     */
+    if (qic117_cmds[ command].cmd_type == motion &&
+        command != QIC_LOGICAL_FORWARD && command != QIC_STOP_TAPE) {
+      location.known = 0;
+    }
+    command_parameter = 0;       /* always turned off for next command */
+    current_command = command;
   }
+  --level;
   TRACE_EXIT;
   return result;
 }
@@ -241,8 +301,8 @@ ftape_command( int command)
 int
 ftape_parameter( int command)
 {
-  check_restrictions = 0;
-  return ftape_command( command);
+  command_parameter = 1;
+  return ftape_command( command + 2);
 }
 
 /*      Wait for the drive to get ready.
@@ -255,6 +315,7 @@ ftape_ready_wait( int timeout, int* status)
   TRACE_FUN( 8, "ftape_ready_wait");
   int result;
   unsigned long t0;
+  const int poll_delay = 100 * MILLISECOND;
 
   for (;;) {
     t0 = jiffies;
@@ -268,19 +329,23 @@ ftape_ready_wait( int timeout, int* status)
       result = 0;
       break;
     }
-    /* this will fail when jiffies wraps around about
-     * once every year :-)
-     */
-    timeout -= ((jiffies - t0) * SECOND) / HZ;
-    if (timeout <= 0) {
-      TRACE( 1, "timeout");
-      result = -ETIME;
-      break;
+    if (timeout >= 0) {
+      /* this will fail when jiffies wraps around about
+       * once every year :-)
+       */
+      timeout -= ((jiffies - t0) * SECOND) / HZ;
+      if (timeout <= 0) {
+        TRACE( 1, "timeout");
+        result = -ETIME;
+        break;
+      }
+      ftape_sleep( poll_delay);
+      timeout -= poll_delay;
+    } else {
+      ftape_sleep( poll_delay);
     }
-    ftape_sleep( SECOND / 10);
-    timeout -= SECOND / 10;
-    if (current->signal & ~current->blocked) {
-      TRACE( 1, "awoken by non-blocked signal :-(");
+    if (current->signal & _NEVER_BLOCK) {
+      TRACE( 1, "interrupted by fatal signal");
       result = -EINTR;
       break;                    /* exit on signal */
     }
@@ -300,15 +365,8 @@ ftape_command_wait( int command, int timeout, int* status)
   /* Drive should be ready, issue command
    */
   result = ftape_command( command);
-  if (result < 0) {
-    TRACE( 1, "command failed");
-  } else {
-    /* Now wait for drive ready or timeout
-     */
+  if (result >= 0) {
     result = ftape_ready_wait( timeout, status);
-    if (result < 0) {
-      TRACE( 1, "ready wait failed");
-    }
   }
   TRACE_EXIT;
   return result;
@@ -317,8 +375,17 @@ ftape_command_wait( int command, int timeout, int* status)
 int
 ftape_parameter_wait( int command, int timeout, int* status)
 {
-  check_restrictions = 0;
-  return ftape_command_wait( command, timeout, status);
+  TRACE_FUN( 8, "ftape_parameter_wait");
+  int result;
+
+  /* Drive should be ready, issue command
+   */
+  result = ftape_parameter( command);
+  if (result >= 0) {
+    result = ftape_ready_wait( timeout, status);
+  }
+  TRACE_EXIT;
+  return result;
 }
 
 /*--------------------------------------------------------------------------
@@ -335,7 +402,7 @@ ftape_report_operation( int *status, int command, int result_length)
   TRACE_FUN( 8, "ftape_report_operation");
   int i, st3;
   int result;
-  unsigned int t0, t1;
+  unsigned int t0, t1, dt;
 
   result = ftape_command( command);
   if (result < 0) {
@@ -344,33 +411,41 @@ ftape_report_operation( int *status, int command, int result_length)
     return result;
   }
   t0 = timestamp();
-  for (i = 0 ; i < 50 ; ++i) {
-    /*  Ack should be asserted within Ttimout + Tack = 8 msec.
-     *  Looks like some drives fail to do this so extend this
-     *  period to 50 msec.
-     */
-    ftape_sleep( MILLISECOND);
+  dt = 0;
+  i = 0;
+  do {
+    ++i;
+    ftape_sleep( 3 * MILLISECOND); /* see remark below */
     result = fdc_sense_drive_status( &st3);
     if (result < 0) {
       TRACE( 1, "fdc_sense_drive_status failed");
       TRACE_EXIT;
       return result;
     }
+    /*  Calculate time difference every iteration because timer may
+     *  wrap around (but only one !) and timediff will account for this.
+     *  Note that the sleep above must be < 1/HZ or we'll lose ticks !
+     */
     t1 = timestamp();
-    if ((st3 & ST3_TRACK_0) != 0) {
-      break;
+    dt += timediff( t0, t1);
+    t0 = t1;
+    /*  Ack should be asserted within Ttimout + Tack = 6 msec.
+     *  Looks like some drives fail to do this so extend this
+     *  period to 300 msec.
+     */
+  } while (!(st3 & ST3_TRACK_0) && dt < 300000);
+  if (st3 & ST3_TRACK_0) {
+    /*  dt may be larger than expected because of other tasks
+     *  scheduled while we were sleeping.
+     */
+    if (i > 1 && dt > 6000) {
+      TRACEx2( 1, "Acknowledge after %u msec. (%i iter)", dt / 1000, i);
     }
-  }
-  if ((st3 & ST3_TRACK_0) == 0) {
+  } else {
+    TRACEx2( 1, "No acknowledge after %u msec. (%i iter)", dt / 1000, i);
     TRACE( 1, "timeout on Acknowledge");
     TRACE_EXIT;
     return -EIO;
-  } else {
-    int dt = timediff( t0, t1);
-
-    if (dt > 8000) {
-      TRACEx1( 4, "Acknowledge within %d usec.", timediff( t0, t1));
-    }
   }
   *status = 0;
   for (i = 0; i < result_length + 1; i++) {
@@ -380,7 +455,14 @@ ftape_report_operation( int *status, int command, int result_length)
       TRACE_EXIT;
       return result;
     }
-    ftape_sleep( 10 * MILLISECOND);
+#if 1
+    /*  fdc_seek does interrupt wait, so why should we ?
+     *  (it will only fail causing fdc to be reset...)
+     *  It's only purpose may be the delay, we'll have to find out!
+     */
+#else
+    fdc_interrupt_wait( 25 * MILLISECOND); /* fails only if hw fails */
+#endif
     result = fdc_sense_drive_status( &st3);
     if (result < 0) {
       TRACE( 1, "fdc_sense_drive_status (2) failed");
@@ -418,8 +500,8 @@ ftape_report_raw_drive_status( int *status)
   if (result < 0) {
     TRACE( 1, "report_operation failed");
     result = -EIO;
-  } else if (!interruptable && (*status & QIC_STATUS_READY)) {
-    interruptable = 1;
+  } else if (*status & QIC_STATUS_READY) {
+    current_command = 0;        /* completed */
   }
   TRACE_EXIT;
   return result;
@@ -466,9 +548,13 @@ ftape_report_error( int* error, int* command, int report)
     *error = code & 0xff;
     *command = (code >> 8) & 0xff;
     if (report) {
-      TRACEi( 3, "errorcode:", *error);
+      if (*error != 0) {
+        TRACEi( 3, "errorcode:", *error);
+      } else {
+        TRACE( 3, "No error");
+      }
     }
-    if (report && tracing > 3) {
+    if (report && *error != 0 && tracing > 3) {
       if (*error >= 0 && *error < NR_ITEMS( ftape_errors)) {
         TRACEx1( -1, "%sFatal ERROR:",
                 (ftape_errors[ *error].fatal ? "" : "Non-"));
@@ -547,7 +633,16 @@ ftape_report_configuration( qic_model* model, int* rate,
       }
       switch (status & QIC_TAPE_LEN_MASK) {
       case QIC_TAPE_205FT:
-        *tape_len = (config & QIC_CONFIG_LONG) ? 425 : 205;
+        /* Unfortunately the new QIC-117 rev G standard shows
+         * no way to discriminate between 205 and 425 ft tapes.
+         * The obvious way seems not to be used: the QIC_CONFIG_LONG
+         * bit isn't used for this (on all drives ?).
+         */
+        if (config & QIC_CONFIG_LONG) {
+          *tape_len = 425;      /* will this ever execute ??? */
+        } else {
+          *tape_len = 0;        /* length unknown: 205 or 425 ft. */
+        }
         break;
       case QIC_TAPE_307FT:
         *tape_len = 307;
@@ -693,7 +788,7 @@ ftape_set_rate_test( int* supported)
   for (i = 0 ; i < NR_ITEMS( rates) ; ++i) {
     result = ftape_command( QIC_SELECT_RATE);
     if (result >= 0) {
-      result = ftape_parameter_wait( rates[ i].drive_code + 2,
+      result = ftape_parameter_wait( rates[ i].drive_code,
                                     1 * SECOND, &status);
       if (result >= 0) {
         if (status & QIC_STATUS_ERROR) {
@@ -734,7 +829,7 @@ ftape_set_data_rate( int new_rate)
         
     result = ftape_command( QIC_SELECT_RATE);
     if (result >= 0) {
-      result = ftape_parameter_wait( rates[ new_rate].drive_code + 2,
+      result = ftape_parameter_wait( rates[ new_rate].drive_code,
                                     1 * SECOND, &status);
       result = ftape_report_raw_drive_status( &status);
       if (result >= 0 && (status & QIC_STATUS_ERROR)) {
@@ -755,15 +850,38 @@ ftape_set_data_rate( int new_rate)
     if (result < 0) {
       TRACEx1( 4, "could not set %sb/s data rate", rates[ new_rate].text);
     } else {
-      TRACEx4( 2, "%s drive with %d ft. QIC-%s tape @ %sb/s",
+      TRACEx2( 2, "%s drive @ %sb/s",
               (model == prehistoric) ? "prehistoric" :
-              ((model == pre_qic117c) ? "pre qic-117c" :
-               ((model == post_qic117b) ? "post qic-117b" : "post qic-117d")),
-              tape_len,         /* 0 == variable ! */
-              (qic_std == QIC_TAPE_QIC40) ? "40" :
-              ((qic_std == QIC_TAPE_QIC80) ? "80" :
-               ((qic_std == QIC_TAPE_QIC3010) ? "3010" : "3020")),
+              ((model == pre_qic117c) ? "pre QIC-117C" :
+               ((model == post_qic117b) ? "post QIC-117B" : "post QIC-117D")),
               rates[ new_rate].text);
+      if (tape_len == 0) {
+        TRACEx1( 2, "unknown length QIC-%s tape",
+                (qic_std == QIC_TAPE_QIC40) ? "40" :
+                ((qic_std == QIC_TAPE_QIC80) ? "80" :
+                 ((qic_std == QIC_TAPE_QIC3010) ? "3010" : "3020")));
+      } else {
+        TRACEx2( 2, "%d ft. QIC-%s tape",
+                tape_len,
+                (qic_std == QIC_TAPE_QIC40) ? "40" :
+                ((qic_std == QIC_TAPE_QIC80) ? "80" :
+                 ((qic_std == QIC_TAPE_QIC3010) ? "3010" : "3020")));
+      }
+      /*
+       *  Set data rate and write precompensation as specified:
+       *
+       *            |  QIC-40/80  | QIC-3010/3020
+       *   rate     |   precomp   |    precomp
+       *  ----------+-------------+--------------
+       *  250 Kbps. |   250 ns.   |     0 ns.
+       *  500 Kbps. |   125 ns.   |     0 ns.
+       *    1 Mbps. |    42 ns.   |     0 ns.
+       */
+      if (qic_std == QIC_TAPE_QIC40 || qic_std == QIC_TAPE_QIC80) {
+        fdc_set_write_precomp( rates[ new_rate].precomp);
+      } else {
+        fdc_set_write_precomp( 0);
+      }
       fdc_set_data_rate( rates[ new_rate].fdc_code);
       ftape_data_rate = new_rate; /* store rate set */
     }
@@ -789,16 +907,17 @@ ftape_seek_head_to_track( int track)
     TRACE( -1, "track out of bounds");
     result = -EINVAL;
   } else {
-    TRACEx1( 4, "seeking track %d", track);
+    TRACEx1( 5, "seeking track %d", track);
     result = ftape_command( QIC_SEEK_HEAD_TO_TRACK);
     if (result < 0) {
       TRACE( 1, "ftape_command failed");
     } else {
-      result = ftape_parameter_wait( track + 2, timeout.head_seek, &status);
+      result = ftape_parameter_wait( track, timeout.head_seek, &status);
       if (result < 0) {
         TRACE( 1, "ftape_parameter_wait failed");
       } else {
         location.track = track;
+        might_be_off_track = 0;
       }
     }
   }
@@ -818,14 +937,14 @@ ftape_wakeup_drive( wake_up_types method)
   case wake_up_colorado:
     result = ftape_command( QIC_PHANTOM_SELECT);    
     if (result == 0) {
-      result = ftape_parameter( 2); /* for tape unit nr 0 */
+      result = ftape_parameter( /* unit */ 0);
     }
     break;
   case wake_up_mountain:
     result = ftape_command( QIC_SOFT_SELECT);
     if (result == 0) {
       ftape_sleep( MILLISECOND);  /* NEEDED */
-      result = ftape_command( 20);
+      result = ftape_parameter( 18);
     }
     break;
   case wake_up_insight:
@@ -878,7 +997,7 @@ ftape_put_drive_to_sleep( vendor_struct drive_type)
 int
 ftape_reset_drive( void)
 {
-  TRACE_FUN( 4, "ftape_reset_drive");
+  TRACE_FUN( 8, "ftape_reset_drive");
   int result = 0;
   int status;
   int err_code;
@@ -921,8 +1040,8 @@ ftape_reset_drive( void)
         result = -EIO;
       }
     }
-    if (current->signal & ~current->blocked) {
-      TRACE( 1, "aborted by signal");
+    if (current->signal & _DONT_BLOCK) {
+      TRACE( 1, "aborted by non-blockable signal");
       result = -EINTR;
       break;                  /* exit on signal */
     }
@@ -937,4 +1056,3 @@ ftape_reset_drive( void)
   TRACE_EXIT;
   return result;
 }
-
