@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 1993,1994 Bas Laarhoven.
+ *      Copyright (C) 1993-1995 Bas Laarhoven.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,11 +15,11 @@ You should have received a copy of the GNU General Public License
 along with this program; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
- $Source: /home/bas/distr/ftape-2.02/RCS/ftape-read.c,v $
+ $Source: /home/bas/distr/ftape-2.03b/RCS/ftape-read.c,v $
  $Author: bas $
  *
- $Revision: 1.23 $
- $Date: 1995/01/15 15:56:46 $
+ $Revision: 1.30 $
+ $Date: 1995/05/27 08:54:21 $
  $State: Beta $
  *
  *      This file contains the reading code
@@ -27,11 +27,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 static char RCSid[] =
-"$Id: ftape-read.c,v 1.23 1995/01/15 15:56:46 bas Beta bas $";
+"$Id: ftape-read.c,v 1.30 1995/05/27 08:54:21 bas Beta $";
 
 
+#include "kernel-version.h"
+#ifdef MODULE
+# include <linux/config.h>
+# include <linux/module.h>
+#endif
 #include <linux/string.h>
 #include <linux/errno.h>
+#include <linux/mm.h>
 #include <asm/segment.h>
 
 #include "ftape.h"
@@ -47,19 +53,18 @@ static char RCSid[] =
 
 /*      Global vars.
  */
-int buf_pos_rd = 0;
-int buf_len_rd = 0;
-int just_before_eof = 0;
 
 /*      Local vars.
  */
+int buf_pos_rd = 0;
+int buf_len_rd = 0;
 
 void
 ftape_zap_read_buffers( void)
 {
   int i;
 
-  for (i = 0; i < NR_ITEMS( buffer); ++i) {
+  for (i = 0; i < ftape_num_buffers; ++i) {
     buffer[ i].address = ((byte*) tape_buffer) + i * BUFF_SIZE;
     buffer[ i].status = waiting;
     buffer[ i].bytes = 0;
@@ -312,19 +317,22 @@ read_segment( unsigned segment_id, byte* address, int* eof_mark,
     }
     /*  Allow escape from this loop on signal !
      */
-    if (current->signal & ~_SIG_BLOCK) {
-      TRACE( 2, "interrupted by signal");
+    if (current->signal & _DONT_BLOCK) {
+      TRACE( 2, "interrupted by non-blockable signal");
       TRACE_EXIT;
       return -EINTR;
     }
     /*    If we got a segment: quit, or else retry up to limit.
-     *    Allow at least three attempts, First causing the error,
-     *    Second for possibly failing (fast) ftape_start_tape, Third
-     *    for recovery of ftape_start_tape.
      */
-    if (read_done || retry > RETRIES_ON_ECC_ERROR) {
+    if (read_done) {
       break;
     }
+    if (retry > RETRIES_ON_ECC_ERROR) {
+      history.defects++;
+      TRACE( 1, "too many retries on ecc failure");
+      TRACE_EXIT;
+      return -ENODATA;
+    }
     /*    Now at least one buffer is empty !
      *    Restart runner & tape if needed.
      */
@@ -374,6 +382,7 @@ read_header_segment( byte* address)
   unsigned int max_floppy_sector;
   int first_failed = 0;
   int status;
+  int new_tape_len;
 
   result = ftape_report_drive_status( &status);
   if (result < 0) {
@@ -404,7 +413,7 @@ read_header_segment( byte* address)
     }
   }
   if (result < 0 || header_segment >= ftape_last_segment.id) {
-    TRACE( 1, "no readable segment found");
+    TRACE( 1, "no readable header segment found");
     TRACE_EXIT;
     return -EIO;
   }
@@ -447,21 +456,49 @@ read_header_segment( byte* address)
   TRACEx6( 4, "(fmt/spt/tpc/fhm/ftm/fsm) = %d/%d/%d/%d/%d/%d",
           format_code, segments_per_track, tracks_per_tape,
           max_floppy_side, max_floppy_track, max_floppy_sector);
+  new_tape_len = tape_len;
   switch (format_code) {
-  case 2:
-  case 5:
+  case fmt_425ft:
+    new_tape_len = 425;
     break;
-  case 3:
-    TRACE( 1, "1100 ft tape : experimental support !");
+  case fmt_normal:
+    if (tape_len == 0) {        /* otherwise 307 ft */
+      new_tape_len = 205;
+    }
     break;
-  case 4:
-    TRACE( 1, "variable length (wide) tape : experimental support !");
+  case fmt_1100ft:
+    new_tape_len = 1100;
     break;
+  case fmt_wide: {
+    int segments_per_1000_inch = 1; /* non-zero default for switch */
+    switch (qic_std) {
+    case QIC_TAPE_QIC40:
+      segments_per_1000_inch = 332;
+      break;
+    case QIC_TAPE_QIC80:
+      segments_per_1000_inch = 488;
+      break;
+    case QIC_TAPE_QIC3010:
+      segments_per_1000_inch = 730;
+      break;
+    case QIC_TAPE_QIC3020:
+      segments_per_1000_inch = 1430;
+      break;
+    }
+    new_tape_len = (1000 * segments_per_track +
+                    (segments_per_1000_inch - 1)) / segments_per_1000_inch;
+    break;
+  }
   default:
     TRACE( 1, "unknown tape format, please report !");
     TRACE_EXIT;
     return -EIO;
   }    
+  if (new_tape_len != tape_len) {
+    tape_len = new_tape_len;
+    TRACEx1( 1, "calculated tape length is %d ft", tape_len);
+    ftape_calc_timeouts();
+  }
   if (segments_per_track == 0 && tracks_per_tape == 0 &&
       max_floppy_side == 0 && max_floppy_track == 0 &&
       max_floppy_sector == 0) {
@@ -505,7 +542,7 @@ read_header_segment( byte* address)
        != max_floppy_side) ||
       (segments_per_head / segments_per_cylinder - 1 != max_floppy_track) ||
       (segments_per_cylinder * SECTORS_PER_SEGMENT != max_floppy_sector)
-#ifndef TESTING
+#ifdef TESTING
       || (format_code == 4 && (max_floppy_track != 254 || max_floppy_sector != 128))
 #endif
       ) {
@@ -599,14 +636,19 @@ _ftape_read( char* buff, int req_len)
       }
       result = read_segment( ftape_seg_pos, deblock_buffer, &eof_mark, 1);
       if (result < 0) {
+        if (result == -ENODATA) {
+          /*  Unable to recover tape data, return error and skip bad spot.
+           */
+          ++ftape_seg_pos;
+        }
         TRACEx1( 4, "read_segment result: %d", result);
         TRACE_EXIT;
         return result;
       }
       /*  Allow escape from this loop on signal !
        */
-      if (current->signal & ~_SIG_BLOCK) {
-        TRACE( 2, "interrupted by signal");
+      if (current->signal & _DONT_BLOCK) {
+        TRACE( 2, "interrupted by non-blockable signal");
         TRACE_EXIT;
         return -EINTR;
       }
@@ -618,21 +660,22 @@ _ftape_read( char* buff, int req_len)
      */
     cnt = (buf_len_rd < to_do) ? buf_len_rd : to_do;
     TRACEi( 7, "nr bytes just read:", cnt);
-    result = verify_area( VERIFY_WRITE, buff, cnt);
-    if (result) {
-      TRACEx1( 1, "verify_area failed, exitcode = %d", result);
-      TRACE_EXIT;
-      return -EIO;
+    if (cnt > 0) {
+      result = verify_area( VERIFY_WRITE, buff, cnt);
+      if (result) {
+        TRACEx1( 1, "verify_area failed, exitcode = %d", result);
+        TRACE_EXIT;
+        return -EIO;
+      }
+      memcpy_tofs( buff, deblock_buffer + buf_pos_rd, cnt);
+      buff += cnt;
+      to_do -= cnt;             /* what's left from req_len */
+      remaining -= cnt;         /* what remains on this tape */
+      bytes_read += cnt;        /* what we got so far */
+      buf_pos_rd += cnt;        /* index in buffer */
+      buf_len_rd -= cnt;        /* remaining bytes in buffer */
     }
-    memcpy_tofs( buff, deblock_buffer + buf_pos_rd, cnt);
-    buff += cnt;
-    to_do -= cnt;               /* what's left from req_len */
-    remaining -= cnt;           /* what remains on this tape */
-    bytes_read += cnt;          /* what we got so far */
-    buf_pos_rd += cnt;          /* index in buffer */
-    buf_len_rd -= cnt;          /* remaining bytes in buffer */
     if (eof_mark && buf_len_rd == 0) { /* nothing left */
-      just_before_eof = 1;
       TRACE( 5, "partial count because of eof mark");
       if (bytes_read == 0) {
         eof_mark = 0;           /* no need for mark next read */
